@@ -41,6 +41,7 @@
 #define SPECIAL_BUILTIN 0x08	/* This is a Posix `special' builtin. */
 #define ASSIGNMENT_BUILTIN 0x10	/* This builtin takes assignment statements. */
 #define POSIX_BUILTIN	0x20	/* This builtins is special in the Posix command search order. */
+#define LOCALVAR_BUILTIN   0x40	/* This builtin creates local variables */
 
 #define BASE_INDENT	4
 
@@ -50,7 +51,7 @@ struct builtin {
   sh_builtin_func_t *function;	/* The address of the invoked function. */
   int flags;			/* One of the #defines above. */
   char * const *long_doc;	/* NULL terminated array of strings. */
-  const char *short_doc;	/* Short version of documenation. */
+  const char *short_doc;	/* Short version of documentation. */
   char *handle;			/* for future use */
 };
 
